@connect /

drop table t;
set echo on

clear screen
create table t
( orig_string       varchar2(30),
  encrypted_string  raw(32),    -- round up to multiple of 16
  decrypted_string  varchar2(30)
)
/
pause

clear screen
DECLARE
   l_string          t.orig_string%type :=  'Hello World';
   l_encrypted_raw   t.encrypted_string%type;
   l_key_bytes_raw   raw(32) default
                     utl_raw.cast_to_raw( 'Secret Key Secret Key Secret Key' );
   l_encrypt_typ     PLS_INTEGER default
                     DBMS_CRYPTO.ENCRYPT_AES256
                     + DBMS_CRYPTO.CHAIN_CBC
                     + DBMS_CRYPTO.PAD_PKCS5;
BEGIN
   l_encrypted_raw := 
    DBMS_CRYPTO.ENCRYPT
    ( src => UTL_I18N.STRING_TO_RAW (l_string,  'AL32UTF8'),
      typ => l_encrypt_typ,
      key => l_key_bytes_raw );

    insert into t ( orig_string, encrypted_string )
    values ( l_string, l_encrypted_raw );
end;
/
pause

column orig_string format a11
column dump format a30
column encrypted_string format a32
clear screen
select orig_string, dump(orig_string,16) dump, encrypted_string
  from t
/
pause

clear screen
select orig_string, decrypted_string
  from t
/
pause
clear screen
DECLARE
   l_string          t.decrypted_string%type;
   l_encrypted_raw   t.encrypted_string%type;
   l_key_bytes_raw   raw(32) default
                     utl_raw.cast_to_raw( 'Secret Key Secret Key Secret Key' );
   l_encrypt_typ     PLS_INTEGER default
                     DBMS_CRYPTO.ENCRYPT_AES256 + DBMS_CRYPTO.CHAIN_CBC + DBMS_CRYPTO.PAD_PKCS5;
BEGIN
    for x in ( select rowid rid, encrypted_string from t )
    loop
        l_string := 
        utl_i18n.raw_to_char( 
              dbms_crypto.decrypt
            ( src => x.encrypted_string,
              typ => l_encrypt_typ,
              key => l_key_bytes_raw ), 'AL32UTF8' );

        update t set decrypted_string = l_string where rowid = x.rid;
    end loop;
end;
/
pause
clear screen
select orig_string, decrypted_string
  from t
/
pause
