connect /
alter system set wallet close;
drop table t;
set echo on
drop tablespace encrypted including contents and datafiles;
drop tablespace clear including contents and datafiles;
set linesize 1000

clear screen
!cat $ORACLE_HOME/network/admin/sqlnet.ora
pause

clear screen
rem ALTER SYSTEM SET ENCRYPTION KEY identified by foobar;
ALTER SYSTEM SET ENCRYPTION WALLET OPEN IDENTIFIED BY foobar;
pause

clear screen
create tablespace encrypted
datafile '/tmp/encrypted.dbf' size 1m
autoextend on next 1m
ENCRYPTION /* using algorithm... */ 
default storage ( ENCRYPT );

create tablespace clear
datafile '/tmp/clear.dbf' size 1m
autoextend on next 1m;
pause

clear screen
create table t
tablespace CLEAR
as
select * 
  from all_users;

create index t_idx 
on t(lower(username))
tablespace CLEAR;
pause

clear screen
alter system checkpoint;
!strings /tmp/clear.dbf | grep -i ops.tkyte
pause

clear screen
alter table t move
tablespace ENCRYPTED;

alter index t_idx rebuild
tablespace ENCRYPTED;
pause


clear screen
alter system checkpoint;
!strings /tmp/encrypted.dbf | grep -i ops.tkyte
pause
!strings /tmp/encrypted.dbf | head
pause

clear screen
set autotrace traceonly explain
select * from t where lower(username) like 'ops$%';
set autotrace off
pause


drop table encrypted;
drop table clear_text;
clear screen
create table encrypted 
tablespace encrypted
CACHE
as 
select * 
  from stage 
 where 1=0;
create table clear_text 
tablespace clear
CACHE
as 
select * 
  from stage 
 where 1=0;
pause

clear screen
declare
    l_start number;
begin
    l_start := dbms_utility.get_cpu_time;
    insert into clear_text
    select *
      from stage;
    dbms_output.put_line( (dbms_utility.get_cpu_time-l_start) || ' hsecs' );
end;
/
/
pause
clear screen
exec show_space( 'CLEAR_TEXT' )
pause

clear screen
declare
    l_start number;
begin
    l_start := dbms_utility.get_cpu_time;
    insert into encrypted
    select *
      from stage;
    dbms_output.put_line( (dbms_utility.get_cpu_time-l_start) || ' hsecs' );
end;
/
/
pause
clear screen
exec show_space( 'ENCRYPTED' )
pause

clear screen
declare
    l_start number := dbms_utility.get_cpu_time;
    l_reads number;
begin
    select b.value into l_reads
      from v$statname a, v$mystat b
     where a.statistic# = b.statistic#
       and a.name = 'physical reads';
    for x in (select /*+ CACHE(clear_text) */ * from clear_text)
    loop
        null;
    end loop;
    select b.value-l_reads into l_reads
      from v$statname a, v$mystat b
     where a.statistic# = b.statistic#
       and a.name = 'physical reads';
    dbms_output.put_line
    ( (dbms_utility.get_cpu_time-l_start) || ' hsecs ' || l_reads || ' reads' );
end;
/
/
pause

clear screen
declare
    l_start number := dbms_utility.get_cpu_time;
    l_reads number;
begin
    select b.value into l_reads
      from v$statname a, v$mystat b
     where a.statistic# = b.statistic#
       and a.name = 'physical reads';
    for x in (select /*+ CACHE(encrypted) */ * from encrypted)
    loop
        null;
    end loop;
    select b.value-l_reads into l_reads
      from v$statname a, v$mystat b
     where a.statistic# = b.statistic#
       and a.name = 'physical reads';
    dbms_output.put_line
    ( (dbms_utility.get_cpu_time-l_start) || ' hsecs ' || l_reads || ' reads' );
end;
/
/
pause


clear screen
declare
    l_start number;
begin
    l_start := dbms_utility.get_cpu_time;
    insert /*+ APPEND */ into clear_text
    select *
      from stage;
    dbms_output.put_line( (dbms_utility.get_cpu_time-l_start) || ' hsecs' );
    commit;
end;
/
/
pause
clear screen
exec show_space( 'CLEAR_TEXT' )
pause

clear screen
declare
    l_start number;
begin
    l_start := dbms_utility.get_cpu_time;
    insert /*+ APPEND */ into encrypted
    select *
      from stage;
    dbms_output.put_line( (dbms_utility.get_cpu_time-l_start) || ' hsecs' );
    commit;
end;
/
/
pause
clear screen
exec show_space( 'ENCRYPTED' )
pause

clear screen
declare
    l_start number := dbms_utility.get_cpu_time;
    l_reads number;
begin
    select b.value into l_reads
      from v$statname a, v$mystat b
     where a.statistic# = b.statistic#
       and a.name = 'physical reads';
    execute immediate 'alter system flush buffer_cache';
    for x in (select * from clear_text)
    loop
        null;
    end loop;
    select b.value-l_reads into l_reads
      from v$statname a, v$mystat b
     where a.statistic# = b.statistic#
       and a.name = 'physical reads';
    dbms_output.put_line
    ( (dbms_utility.get_cpu_time-l_start) || ' hsecs ' || l_reads || ' reads' );
end;
/
/
pause

clear screen
declare
    l_start number := dbms_utility.get_cpu_time;
    l_reads number;
begin
    select b.value into l_reads
      from v$statname a, v$mystat b
     where a.statistic# = b.statistic#
       and a.name = 'physical reads';
    execute immediate 'alter system flush buffer_cache';
    for x in (select * from encrypted)
    loop
        null;
    end loop;
    select b.value-l_reads into l_reads
      from v$statname a, v$mystat b
     where a.statistic# = b.statistic#
       and a.name = 'physical reads';
    dbms_output.put_line
    ( (dbms_utility.get_cpu_time-l_start) || ' hsecs ' || l_reads || ' reads' );
end;
/
/
pause
